package org.apache.logging.log4j.core.impl;

import java.net.URI;
import org.apache.logging.log4j.core.LifeCycle;
import org.apache.logging.log4j.core.LoggerContext;
import org.apache.logging.log4j.core.config.Configuration;
import org.apache.logging.log4j.core.config.ConfigurationFactory;
import org.apache.logging.log4j.core.config.ConfigurationSource;
import org.apache.logging.log4j.core.selector.ClassLoaderContextSelector;
import org.apache.logging.log4j.core.selector.ContextSelector;
import org.apache.logging.log4j.core.util.Loader;
import org.apache.logging.log4j.spi.LoggerContextFactory;
import org.apache.logging.log4j.status.StatusLogger;
import org.apache.logging.log4j.util.PropertiesUtil;

public class Log4jContextFactory implements LoggerContextFactory {
   private static final StatusLogger LOGGER = StatusLogger.getLogger();
   private ContextSelector selector;

   public Log4jContextFactory() {
      String sel = PropertiesUtil.getProperties().getStringProperty("Log4jContextSelector");
      if (sel != null) {
         try {
            this.selector = (ContextSelector)Loader.newCheckedInstanceOf(sel, ContextSelector.class);
         } catch (Exception var3) {
            LOGGER.error("Unable to create context {}", new Object[]{sel, var3});
         }
      }

      if (this.selector == null) {
         this.selector = new ClassLoaderContextSelector();
      }

   }

   public LoggerContext getContext(String fqcn, ClassLoader loader, Object externalContext, boolean currentContext) {
      LoggerContext ctx = this.selector.getContext(fqcn, loader, currentContext);
      ctx.setExternalContext(externalContext);
      if (ctx.getState() == LifeCycle.State.INITIALIZED) {
         ctx.start();
      }

      return ctx;
   }

   public LoggerContext getContext(String fqcn, ClassLoader loader, Object externalContext, boolean currentContext, ConfigurationSource source) {
      LoggerContext ctx = this.selector.getContext(fqcn, loader, currentContext, (URI)null);
      if (externalContext != null && ctx.getExternalContext() == null) {
         ctx.setExternalContext(externalContext);
      }

      if (ctx.getState() == LifeCycle.State.INITIALIZED) {
         if (source != null) {
            ContextAnchor.THREAD_CONTEXT.set(ctx);
            Configuration config = ConfigurationFactory.getInstance().getConfiguration(source);
            LOGGER.debug("Starting LoggerContext[name={}] from configuration {}", new Object[]{ctx.getName(), source});
            ctx.start(config);
            ContextAnchor.THREAD_CONTEXT.remove();
         } else {
            ctx.start();
         }
      }

      return ctx;
   }

   public LoggerContext getContext(String fqcn, ClassLoader loader, Object externalContext, boolean currentContext, URI configLocation, String name) {
      LoggerContext ctx = this.selector.getContext(fqcn, loader, currentContext, configLocation);
      if (externalContext != null && ctx.getExternalContext() == null) {
         ctx.setExternalContext(externalContext);
      }

      if (ctx.getState() == LifeCycle.State.INITIALIZED) {
         if (configLocation == null && name == null) {
            ctx.start();
         } else {
            ContextAnchor.THREAD_CONTEXT.set(ctx);
            Configuration config = ConfigurationFactory.getInstance().getConfiguration(name, configLocation);
            LOGGER.debug("Starting LoggerContext[name={}] from configuration at {}", new Object[]{ctx.getName(), configLocation});
            ctx.start(config);
            ContextAnchor.THREAD_CONTEXT.remove();
         }
      }

      return ctx;
   }

   public ContextSelector getSelector() {
      return this.selector;
   }

   public void removeContext(org.apache.logging.log4j.spi.LoggerContext context) {
      if (context instanceof LoggerContext) {
         this.selector.removeContext((LoggerContext)context);
      }

   }
}
