package org.apache.logging.log4j.core.layout;

import java.io.IOException;
import java.io.InterruptedIOException;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.lang.management.ManagementFactory;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
import org.apache.logging.log4j.core.config.plugins.PluginBuilderAttribute;
import org.apache.logging.log4j.core.config.plugins.PluginBuilderFactory;
import org.apache.logging.log4j.core.config.plugins.PluginFactory;
import org.apache.logging.log4j.core.util.Charsets;
import org.apache.logging.log4j.core.util.Constants;
import org.apache.logging.log4j.core.util.Transform;

@Plugin(
   name = "HtmlLayout",
   category = "Core",
   elementType = "layout",
   printObject = true
)
public final class HtmlLayout extends AbstractStringLayout {
   private static final int BUF_SIZE = 256;
   private static final String TRACE_PREFIX = "<br />&nbsp;&nbsp;&nbsp;&nbsp;";
   private static final String REGEXP;
   private static final String DEFAULT_TITLE = "Log4j Log Messages";
   private static final String DEFAULT_CONTENT_TYPE = "text/html";
   public static final String DEFAULT_FONT_FAMILY = "arial,sans-serif";
   private final long jvmStartTime;
   private final boolean locationInfo;
   private final String title;
   private final String contentType;
   private final String font;
   private final String fontSize;
   private final String headerSize;

   private HtmlLayout(boolean locationInfo, String title, String contentType, Charset charset, String font, String fontSize, String headerSize) {
      super(charset);
      this.jvmStartTime = ManagementFactory.getRuntimeMXBean().getStartTime();
      this.locationInfo = locationInfo;
      this.title = title;
      this.contentType = this.addCharsetToContentType(contentType);
      this.font = font;
      this.fontSize = fontSize;
      this.headerSize = headerSize;
   }

   private String addCharsetToContentType(String contentType) {
      if (contentType == null) {
         return "text/html; charset=" + this.getCharset();
      } else {
         return contentType.contains("charset") ? contentType : contentType + "; charset=" + this.getCharset();
      }
   }

   public String toSerializable(LogEvent event) {
      StringBuilder sbuf = new StringBuilder(256);
      sbuf.append(Constants.LINE_SEPARATOR).append("<tr>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<td>");
      sbuf.append(event.getTimeMillis() - this.jvmStartTime);
      sbuf.append("</td>").append(Constants.LINE_SEPARATOR);
      String escapedThread = Transform.escapeHtmlTags(event.getThreadName());
      sbuf.append("<td title=\"").append(escapedThread).append(" thread\">");
      sbuf.append(escapedThread);
      sbuf.append("</td>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<td title=\"Level\">");
      if (event.getLevel().equals(Level.DEBUG)) {
         sbuf.append("<font color=\"#339933\">");
         sbuf.append(Transform.escapeHtmlTags(String.valueOf(event.getLevel())));
         sbuf.append("</font>");
      } else if (event.getLevel().isMoreSpecificThan(Level.WARN)) {
         sbuf.append("<font color=\"#993300\"><strong>");
         sbuf.append(Transform.escapeHtmlTags(String.valueOf(event.getLevel())));
         sbuf.append("</strong></font>");
      } else {
         sbuf.append(Transform.escapeHtmlTags(String.valueOf(event.getLevel())));
      }

      sbuf.append("</td>").append(Constants.LINE_SEPARATOR);
      String escapedLogger = Transform.escapeHtmlTags(event.getLoggerName());
      if (escapedLogger.isEmpty()) {
         escapedLogger = "root";
      }

      sbuf.append("<td title=\"").append(escapedLogger).append(" logger\">");
      sbuf.append(escapedLogger);
      sbuf.append("</td>").append(Constants.LINE_SEPARATOR);
      if (this.locationInfo) {
         StackTraceElement element = event.getSource();
         sbuf.append("<td>");
         sbuf.append(Transform.escapeHtmlTags(element.getFileName()));
         sbuf.append(':');
         sbuf.append(element.getLineNumber());
         sbuf.append("</td>").append(Constants.LINE_SEPARATOR);
      }

      sbuf.append("<td title=\"Message\">");
      sbuf.append(Transform.escapeHtmlTags(event.getMessage().getFormattedMessage()).replaceAll(REGEXP, "<br />"));
      sbuf.append("</td>").append(Constants.LINE_SEPARATOR);
      sbuf.append("</tr>").append(Constants.LINE_SEPARATOR);
      if (event.getContextStack() != null && !event.getContextStack().isEmpty()) {
         sbuf.append("<tr><td bgcolor=\"#EEEEEE\" style=\"font-size : ").append(this.fontSize);
         sbuf.append(";\" colspan=\"6\" ");
         sbuf.append("title=\"Nested Diagnostic Context\">");
         sbuf.append("NDC: ").append(Transform.escapeHtmlTags(event.getContextStack().toString()));
         sbuf.append("</td></tr>").append(Constants.LINE_SEPARATOR);
      }

      if (event.getContextMap() != null && !event.getContextMap().isEmpty()) {
         sbuf.append("<tr><td bgcolor=\"#EEEEEE\" style=\"font-size : ").append(this.fontSize);
         sbuf.append(";\" colspan=\"6\" ");
         sbuf.append("title=\"Mapped Diagnostic Context\">");
         sbuf.append("MDC: ").append(Transform.escapeHtmlTags(event.getContextMap().toString()));
         sbuf.append("</td></tr>").append(Constants.LINE_SEPARATOR);
      }

      Throwable throwable = event.getThrown();
      if (throwable != null) {
         sbuf.append("<tr><td bgcolor=\"#993300\" style=\"color:White; font-size : ").append(this.fontSize);
         sbuf.append(";\" colspan=\"6\">");
         this.appendThrowableAsHtml(throwable, sbuf);
         sbuf.append("</td></tr>").append(Constants.LINE_SEPARATOR);
      }

      return sbuf.toString();
   }

   public Map<String, String> getContentFormat() {
      return new HashMap();
   }

   public String getContentType() {
      return this.contentType;
   }

   private void appendThrowableAsHtml(Throwable throwable, StringBuilder sbuf) {
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw);

      try {
         throwable.printStackTrace(pw);
      } catch (RuntimeException var10) {
      }

      pw.flush();
      LineNumberReader reader = new LineNumberReader(new StringReader(sw.toString()));
      ArrayList lines = new ArrayList();

      try {
         for(String line = reader.readLine(); line != null; line = reader.readLine()) {
            lines.add(line);
         }
      } catch (IOException var11) {
         if (var11 instanceof InterruptedIOException) {
            Thread.currentThread().interrupt();
         }

         lines.add(var11.toString());
      }

      boolean first = true;
      Iterator i$ = lines.iterator();

      while(i$.hasNext()) {
         String line = (String)i$.next();
         if (!first) {
            sbuf.append("<br />&nbsp;&nbsp;&nbsp;&nbsp;");
         } else {
            first = false;
         }

         sbuf.append(Transform.escapeHtmlTags(line));
         sbuf.append(Constants.LINE_SEPARATOR);
      }

   }

   public byte[] getHeader() {
      StringBuilder sbuf = new StringBuilder();
      sbuf.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" ");
      sbuf.append("\"http://www.w3.org/TR/html4/loose.dtd\">");
      sbuf.append(Constants.LINE_SEPARATOR);
      sbuf.append("<html>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<head>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<meta charset=\"").append(this.getCharset()).append("\"/>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<title>").append(this.title).append("</title>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<style type=\"text/css\">").append(Constants.LINE_SEPARATOR);
      sbuf.append("<!--").append(Constants.LINE_SEPARATOR);
      sbuf.append("body, table {font-family:").append(this.font).append("; font-size: ");
      sbuf.append(this.headerSize).append(";}").append(Constants.LINE_SEPARATOR);
      sbuf.append("th {background: #336699; color: #FFFFFF; text-align: left;}").append(Constants.LINE_SEPARATOR);
      sbuf.append("-->").append(Constants.LINE_SEPARATOR);
      sbuf.append("</style>").append(Constants.LINE_SEPARATOR);
      sbuf.append("</head>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<body bgcolor=\"#FFFFFF\" topmargin=\"6\" leftmargin=\"6\">").append(Constants.LINE_SEPARATOR);
      sbuf.append("<hr size=\"1\" noshade>").append(Constants.LINE_SEPARATOR);
      sbuf.append("Log session start time " + new Date() + "<br>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<br>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<table cellspacing=\"0\" cellpadding=\"4\" border=\"1\" bordercolor=\"#224466\" width=\"100%\">");
      sbuf.append(Constants.LINE_SEPARATOR);
      sbuf.append("<tr>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<th>Time</th>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<th>Thread</th>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<th>Level</th>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<th>Logger</th>").append(Constants.LINE_SEPARATOR);
      if (this.locationInfo) {
         sbuf.append("<th>File:Line</th>").append(Constants.LINE_SEPARATOR);
      }

      sbuf.append("<th>Message</th>").append(Constants.LINE_SEPARATOR);
      sbuf.append("</tr>").append(Constants.LINE_SEPARATOR);
      return sbuf.toString().getBytes(this.getCharset());
   }

   public byte[] getFooter() {
      StringBuilder sbuf = new StringBuilder();
      sbuf.append("</table>").append(Constants.LINE_SEPARATOR);
      sbuf.append("<br>").append(Constants.LINE_SEPARATOR);
      sbuf.append("</body></html>");
      return sbuf.toString().getBytes(this.getCharset());
   }

   @PluginFactory
   public static HtmlLayout createLayout(@PluginAttribute(value = "locationInfo",defaultBoolean = false) boolean locationInfo, @PluginAttribute(value = "title",defaultString = "Log4j Log Messages") String title, @PluginAttribute("contentType") String contentType, @PluginAttribute(value = "charset",defaultString = "UTF-8") Charset charset, @PluginAttribute("fontSize") String fontSize, @PluginAttribute(value = "fontName",defaultString = "arial,sans-serif") String font) {
      HtmlLayout.FontSize fs = HtmlLayout.FontSize.getFontSize(fontSize);
      fontSize = fs.getFontSize();
      String headerSize = fs.larger().getFontSize();
      if (contentType == null) {
         contentType = "text/html; charset=" + charset;
      }

      return new HtmlLayout(locationInfo, title, contentType, charset, font, fontSize, headerSize);
   }

   public static HtmlLayout createDefaultLayout() {
      return newBuilder().build();
   }

   @PluginBuilderFactory
   public static HtmlLayout.Builder newBuilder() {
      return new HtmlLayout.Builder();
   }

   // $FF: synthetic method
   HtmlLayout(boolean x0, String x1, String x2, Charset x3, String x4, String x5, String x6, Object x7) {
      this(x0, x1, x2, x3, x4, x5, x6);
   }

   static {
      REGEXP = Constants.LINE_SEPARATOR.equals("\n") ? "\n" : Constants.LINE_SEPARATOR + "|\n";
   }

   public static class Builder implements org.apache.logging.log4j.core.util.Builder<HtmlLayout> {
      @PluginBuilderAttribute
      private boolean locationInfo;
      @PluginBuilderAttribute
      private String title;
      @PluginBuilderAttribute
      private String contentType;
      @PluginBuilderAttribute
      private Charset charset;
      @PluginBuilderAttribute
      private HtmlLayout.FontSize fontSize;
      @PluginBuilderAttribute
      private String fontName;

      private Builder() {
         this.locationInfo = false;
         this.title = "Log4j Log Messages";
         this.contentType = null;
         this.charset = Charsets.UTF_8;
         this.fontSize = HtmlLayout.FontSize.SMALL;
         this.fontName = "arial,sans-serif";
      }

      public HtmlLayout.Builder withLocationInfo(boolean locationInfo) {
         this.locationInfo = locationInfo;
         return this;
      }

      public HtmlLayout.Builder withTitle(String title) {
         this.title = title;
         return this;
      }

      public HtmlLayout.Builder withContentType(String contentType) {
         this.contentType = contentType;
         return this;
      }

      public HtmlLayout.Builder withCharset(Charset charset) {
         this.charset = charset;
         return this;
      }

      public HtmlLayout.Builder withFontSize(HtmlLayout.FontSize fontSize) {
         this.fontSize = fontSize;
         return this;
      }

      public HtmlLayout.Builder withFontName(String fontName) {
         this.fontName = fontName;
         return this;
      }

      public HtmlLayout build() {
         if (this.contentType == null) {
            this.contentType = "text/html; charset=" + this.charset;
         }

         return new HtmlLayout(this.locationInfo, this.title, this.contentType, this.charset, this.fontName, this.fontSize.getFontSize(), this.fontSize.larger().getFontSize());
      }

      // $FF: synthetic method
      Builder(Object x0) {
         this();
      }
   }

   public static enum FontSize {
      SMALLER("smaller"),
      XXSMALL("xx-small"),
      XSMALL("x-small"),
      SMALL("small"),
      MEDIUM("medium"),
      LARGE("large"),
      XLARGE("x-large"),
      XXLARGE("xx-large"),
      LARGER("larger");

      private final String size;

      private FontSize(String size) {
         this.size = size;
      }

      public String getFontSize() {
         return this.size;
      }

      public static HtmlLayout.FontSize getFontSize(String size) {
         HtmlLayout.FontSize[] arr$ = values();
         int len$ = arr$.length;

         for(int i$ = 0; i$ < len$; ++i$) {
            HtmlLayout.FontSize fontSize = arr$[i$];
            if (fontSize.size.equals(size)) {
               return fontSize;
            }
         }

         return SMALL;
      }

      public HtmlLayout.FontSize larger() {
         return this.ordinal() < XXLARGE.ordinal() ? values()[this.ordinal() + 1] : this;
      }
   }
}
